// OpenCVTest.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"

#include <stdio.h>
#include <iostream>
#include <vector>
#include <opencv2/opencv.hpp>

#ifdef _DEBUG
#pragma comment(lib, "opencv_core230d.lib")
#pragma comment(lib, "opencv_imgproc230d.lib")
#pragma comment(lib, "opencv_highgui230d.lib")
#pragma comment(lib, "opencv_features2d230d.lib")
#pragma comment(lib, "opencv_flann230d.lib")
#pragma comment(lib, "opencv_contrib230d.lib")
#pragma comment(lib, "opencv_objdetect230d.lib")
#pragma comment(lib, "opencv_calib3d230d.lib")

#else
#pragma comment(lib, "opencv_core230.lib")
#pragma comment(lib, "opencv_imgproc230.lib")
#pragma comment(lib, "opencv_highgui230.lib")
#pragma comment(lib, "opencv_features2d230.lib")
#pragma comment(lib, "opencv_flann230.lib")
#pragma comment(lib, "opencv_contrib230.lib")
#pragma comment(lib, "opencv_objdetect230.lib")
#pragma comment(lib, "opencv_video230.lib")
#pragma comment(lib, "opencv_calib3d230.lib")
#pragma comment(lib, "opencv_video230.lib")
#pragma comment(lib, "opencv_gpu230.lib")
#pragma comment(lib, "opencv_legacy230.lib")
#pragma comment(lib, "opencv_ml230.lib")

#endif

using namespace std;
using namespace cv;

int matchesCount = 0;

void help()
{
	printf("\nThis program demonstrates using features2d detector, descriptor extractor and simple matcher\n"
		"Using the SURF desriptor:\n"
		"\n"
		"Usage:\n matcher_simple <image1> <image2>\n");
}

void matches2points(const vector<DMatch>& matches, const vector<KeyPoint>& kpts_train,
					const vector<KeyPoint>& kpts_query, vector<Point2f>& pts_train, vector<Point2f>& pts_query)
{
	pts_train.clear();
	pts_query.clear();
	pts_train.reserve(matches.size());
	pts_query.reserve(matches.size());
	for (size_t i = 0; i < matches.size(); i++)
	{
		const DMatch& match = matches[i];
		pts_query.push_back(kpts_query[match.queryIdx].pt);
		pts_train.push_back(kpts_train[match.trainIdx].pt);
	}

}

double match(const vector<KeyPoint>& /*kpts_train*/, const vector<KeyPoint>& /*kpts_query*/, DescriptorMatcher& matcher,
			 const Mat& train, const Mat& query, vector<DMatch>& matches)
{

	double t = (double)getTickCount();
	matcher.match(query, train, matches); //Using features2d
	return ((double)getTickCount() - t) / getTickFrequency();
}

// ð  Լ
void stopWatch()
{
	static double time = 0.0;
	static bool bCheckTime = false;
	static int timerCount = 0;

	if( !bCheckTime ) {
		time = (double)getTickCount();
		timerCount++;
	}

	if( bCheckTime ) {
		time = ((double)getTickCount() - time) / getTickFrequency();
		cout << "Time" << timerCount << ": " << time << " sec" << endl;
	}
	
	bCheckTime = !bCheckTime;
}

int main(int argc, char** argv)
{
	argc = 3;
	argv[2] = "q1.jpg";
	argv[1] = "21.jpg";
	if(argc != 3)
	{
		help();
		return -1;
	}

	Mat img1 = imread(argv[1], CV_LOAD_IMAGE_GRAYSCALE);
	Mat img2 = imread(argv[2], CV_LOAD_IMAGE_GRAYSCALE);
	if(img1.empty() || img2.empty())
	{
		printf("Can't read one of the images\n");
		return -1;
	}

	cout << "SURF" << endl;
	// detecting keypoints
	SurfFeatureDetector detector(400);
	vector<KeyPoint> keypoints1, keypoints2;
	stopWatch();
	detector.detect(img1, keypoints1);
	detector.detect(img2, keypoints2);
	stopWatch();

	// computing descriptors
	SurfDescriptorExtractor extractor;
	Mat descriptors1, descriptors2;

	stopWatch();
	extractor.compute(img1, keypoints1, descriptors1);
	extractor.compute(img2, keypoints2, descriptors2);
	stopWatch();

	stopWatch();
	// matching descriptors
	BruteForceMatcher<L2<float> > matcher;
	//BruteForceMatcher<Hamming> matcher;
	vector<DMatch> matches;
	match(keypoints1, keypoints2, matcher, descriptors1, descriptors2, matches);
	stopWatch();

	vector<Point2f> mpts_1, mpts_2;
	matches2points(matches, keypoints1, keypoints2, mpts_1, mpts_2); //Extract a list of the (x,y) location of the matches
	vector<uchar> outlier_mask;

	Mat H;
	stopWatch();
	H = findHomography(mpts_1, mpts_2, RANSAC, 3, outlier_mask);
	stopWatch();


	for( size_t m = 0; m < matches.size(); m++ )
	{
		//int i1 = matches[m].queryIdx;
		//int i2 = matches[m].trainIdx;
		if( outlier_mask.empty() || outlier_mask[m] ) {
			matchesCount ++;
		}
	}

	cout << "matches points : " << matchesCount << endl;

	// drawing the results
	if( matchesCount > 20 ) {
		namedWindow("SURF", 1);
		Mat img_matches;
		drawMatches(img2, keypoints2, img1, keypoints1, matches, img_matches, Scalar(0,255,0), Scalar(0,0,255), reinterpret_cast<const vector<char>&> (outlier_mask), 2);
			
		imshow("SURF", img_matches);

		Mat warped;
		Mat diff;
		warpPerspective(img1, warped, H, img2.size());
		imshow("warped", warped);
		absdiff(img2, warped, diff);
		imshow("diff", diff);
	}
	else{
		cout << "rejected. \n";
		imshow("image", img1);
	}

	waitKey();

	waitKey(0);

	return 0;
}
